<?php

namespace Kylin\Activity\Contracts;

use Closure;
use Kylin\Activity\EventLogBag;

/**
 * 可记录管道接口
 *
 * 定义活动日志处理管道中各环节的契约。
 * 实现类通过 handle() 方法对 EventLogBag 进行链式处理。
 */
interface LoggablePipe
{
    public function handle(EventLogBag $event, Closure $next): EventLogBag;
}
