<?php

namespace Kylin\Activity\Exceptions;

use Exception;

/**
 * 无法记录活动日志异常
 *
 * 当无法确定操作者（causer）或日志记录失败时抛出。
 */
class CouldNotLogActivity extends Exception
{
    public static function couldNotDetermineUser($id): self
    {
        return new static("Could not determine a user with identifier `{$id}`.");
    }
}
